# module doc string
"""
This module process the data to prepare for the skills extraction algorithm.
author: - Lois Ji
        - Leo Li
date: Feb 28, 2022
"""

import pandas as pd
import yaml
import pickle

from utils import df_to_dict

params = yaml.safe_load(open("params.yaml"))
short_alias_skill_context_table_output = params["load_data"]["short_alias_skill_context_table_output"]
short_alias_skill_stem_table_output = params["load_data"]["short_alias_skill_stem_table_output"]
stem_alias_table_output = params["load_data"]["stem_alias_table_output"]
context_alias_table_output = params["load_data"]["context_alias_table_output"]
stem_to_skill_mapping_table_output = params["load_data"]["stem_to_skill_mapping_table_output"]
skill_to_context_mapping_table_output = params["load_data"]["skill_to_context_mapping_table_output"]
context_indept_skill_table_output = params["load_data"]["context_indept_skill_table_output"]
ccs_skill_table_output = params["load_data"]["ccs_skill_table_output"]
app_skill_table_output = params["load_data"]["app_skill_table_output"]
pickle_output = params["process_data"]["pickle_output"]


def create_stem_dic(short_alias_path=short_alias_skill_stem_table_output,
                    stem_alias_table_path=stem_alias_table_output):
    """
    function to create dictionaries for skills stems, with both short and long aliases
    :param short_alias_path: data path of the short alias table
    :param stem_alias_table_path: data path of the stem alias table
    :return:
        stem_flashtext_dict: stem dictionary for keywords-matching with word boundary conditions
        stemsDict: stem dictionary with fuzzy-matching logic
    """
    get_short_alias_skill = pd.read_csv(short_alias_path)
    stem_alias = pd.read_csv(stem_alias_table_path)
    stem_alias_short = stem_alias.copy()
    stem_alias_short = stem_alias_short[
        stem_alias_short.skill_component_id.isin(list(get_short_alias_skill.skill_component_id))]
    stem_flashtext_dict = df_to_dict(stem_alias_short.skill_component_stem, stem_alias_short.skill_component_alias)
    stem_alias_long = stem_alias.copy()
    stem_alias_long = stem_alias_long[
        ~stem_alias_long.skill_component_id.isin(list(get_short_alias_skill.skill_component_id))]
    stemsDict = df_to_dict(stem_alias_long.skill_component_stem, stem_alias_long.skill_component_alias)

    return stem_flashtext_dict, stemsDict


def create_context_dic(context_alias_path=context_alias_table_output,
                       short_alias_path=short_alias_skill_context_table_output):
    """
    function to create dictionaries for skills context, with both short and long aliases
    :param context_alias_path: data path of the context alias table
    :param short_alias_path: data path of the short alias table
    :return:
        context_flashtext_dict: context dictionary for keywords-matching with word boundary conditions
        contextsDict: context dictionary with fuzzy-matching logic
    """
    context_alias = pd.read_csv(context_alias_path)
    get_short_alias_skill = pd.read_csv(short_alias_path)
    context_alias_short = context_alias.copy()
    context_alias_short = context_alias_short[
        context_alias_short.skill_component_id.isin(list(get_short_alias_skill.skill_component_id))]
    context_flashtext_dict = df_to_dict(context_alias_short.skill_component_context,
                                        context_alias_short.skill_component_alias)
    context_alias_long = context_alias.copy()
    context_alias_long = context_alias_long[
        ~context_alias_long.skill_component_id.isin(list(get_short_alias_skill.skill_component_id))]
    contextsDict = df_to_dict(context_alias_long.skill_component_context, context_alias_long.skill_component_alias)

    return context_flashtext_dict, contextsDict


def create_stem_to_skill_dic(stem_to_skill_path=stem_to_skill_mapping_table_output):
    """
    function to create stem-to-skill dictionary
    :param stem_to_skill_path: data path to the stem to skills mapping table
    :return:
        stemToSkillDict: stem-to-skill dictionary
    """
    stem_to_skill_mapping = pd.read_csv(stem_to_skill_path)
    stemToSkillDict = df_to_dict(stem_to_skill_mapping.skill_component_stem, stem_to_skill_mapping.skill_title)

    return stemToSkillDict


def create_skill_to_context_dic(skill_to_context_path=skill_to_context_mapping_table_output):
    """
    function to create skill-to-context dictionary
    :param skill_to_context_path: data path to the skill to context mapping table
    :return:
        skillToContextDict: skill-to-context dictionary
    """
    skill_to_context_mapping = pd.read_csv(skill_to_context_path)
    skillToContextDict = df_to_dict(skill_to_context_mapping.skill_title,
                                    skill_to_context_mapping.skill_component_context)
    return skillToContextDict


def create_context_indept_skill(context_indept_skill_path=context_indept_skill_table_output):
    """
    function to create context independent skills set
    :param context_indept_skill_path:
    :return:
        skillSetIndptContext: context independent skills set
    """
    context_indept_skill = pd.read_csv(context_indept_skill_path)
    skillSetIndptContext = set(context_indept_skill.skill_title)

    return skillSetIndptContext


def create_ccs_skill_dic(ccs_skill_path=ccs_skill_table_output):
    """
    function to create ccs skills dictionary
    :param ccs_skill_path: data path to the ccs skills alias table
    :return:
        ccs_flashtext_dict: ccs skills alias dictionary for flashtext extraction
    """
    ccs_skill = pd.read_csv(ccs_skill_path)
    ccs_flashtext_dict = df_to_dict(ccs_skill.skill_title, ccs_skill.skill_component_alias)
    return ccs_flashtext_dict


def create_app_tool_dic(app_tool_path=app_skill_table_output):
    """
    function to create apps and tools dictionary
    :param app_tool_path: data path to the apps and tools skills alias table
    :return:
        app_tool_flashtext_dict: apps and tools alias dictionary for flashtext extraction
    """
    app_tool = pd.read_csv(app_tool_path)
    app_tool_flashtext_dict = df_to_dict(app_tool.skill_title, app_tool.skill_component_alias)
    return app_tool_flashtext_dict


def main(pickle_path=pickle_output):
    """
    function to execute dictionary/set creations, save as a pickle dump
    :param pickle_path: output path of the pickle dump
    :return: None
    """
    stem_flashtext_dict, stemsDict = create_stem_dic()
    context_flashtext_dict, contextsDict = create_context_dic()
    ccs_flashtext_dict = create_ccs_skill_dic()
    app_tool_flashtext_dict = create_app_tool_dic()
    stemToSkillDict = create_stem_to_skill_dic()
    skillToContextDict = create_skill_to_context_dic()
    skillSetIndptContext = create_context_indept_skill()

    dict_list = [stem_flashtext_dict, stemsDict, context_flashtext_dict, contextsDict,
                 stemToSkillDict, skillToContextDict, skillSetIndptContext,
                 ccs_flashtext_dict, app_tool_flashtext_dict]

    file = open(pickle_path, 'wb')
    for dict in dict_list:
        pickle.dump(dict, file)
    file.close()


if __name__ == '__main__':
    main()



# module doc string
"""
This module extracts skills from free text and output them in accordance to SST skill terms
author: - Lois Ji
        - Leo Li
date: Mar 1, 2022
"""
import pickle
import yaml
from ssg_sea.utils import *
from ssg_sea.config.core import CONFIG_FILE_PATH, PICKLE_FILE_PATH

from flashtext import KeywordProcessor

params = yaml.safe_load(open(CONFIG_FILE_PATH))
pickle_input = PICKLE_FILE_PATH

# Load data artefacts
file = open(pickle_input, 'rb')
stem_flashtext_dict = pickle.load(file)
stemsDict = pickle.load(file)
context_flashtext_dict = pickle.load(file)
contextsDict = pickle.load(file)
stemToSkillDict = pickle.load(file)
skillToContextDict = pickle.load(file)
skillSetIndptContext = pickle.load(file)
ccs_flashtext_dict = pickle.load(file)
app_tool_flashtext_dict = pickle.load(file)
file.close()

# Load KeywordProcessors
context_keyword_processor = KeywordProcessor()
context_keyword_processor.add_keywords_from_dict(context_flashtext_dict)
stem_keyword_processor = KeywordProcessor()
stem_keyword_processor.add_keywords_from_dict(stem_flashtext_dict)
app_keyword_processor = KeywordProcessor()
app_keyword_processor.add_keywords_from_dict(app_tool_flashtext_dict)
ccs_keyword_processor = KeywordProcessor()
ccs_keyword_processor.add_keywords_from_dict(ccs_flashtext_dict)


def extract_skills(text):
    """
    output the extraction results of the skills extraction algorithm
    :param text: input text for skills extraction
    :return:
        output_list: list of skills with weight and tpye
    """
    context_list = extractionFrTxt(text, flashProcessor=context_keyword_processor,
                                   usedDict=contextsDict)
    stem_list = extractionFrTxt(text, flashProcessor=stem_keyword_processor,
                                usedDict=stemsDict)
    app_word_list = extractionFrTxt(text, flashProcessor=app_keyword_processor,
                                    usedDict="noDict")
    ccs_word_list = extractionFrTxt(text, flashProcessor=ccs_keyword_processor,
                                    usedDict="noDict")

    tsc_list = extractTSC(context_list=context_list, stem_list=stem_list,
                          skill_to_context_dict=skillToContextDict, stem_to_skill_dict=stemToSkillDict,
                          skill_indpt_context_set=skillSetIndptContext)
    app_list = extractAPP(app_word_list)
    ccs_list = extractCCS(ccs_word_list)

    tsc_output = getSkillWeight(skillExtractionList=tsc_list, skillForm="TSC")
    app_output = getSkillWeight(skillExtractionList=app_list, skillForm="App/Tools")
    ccs_output = getSkillWeight(skillExtractionList=ccs_list, skillForm="CCS")

    output_list = tsc_output + app_output + ccs_output

    results = {}
    _list = []
    keys = range(len(output_list))

    for tuple in output_list:
        extractDict = {
            "skill name": tuple[0],
            "skill weight": tuple[1],
            "skill form": tuple[2]
        }
        _list.append(extractDict)

    for i in keys:
        results[i] = _list[i]

    if len(results) == 0:
        output = {}
    else:
        output = {
            "extractions": results
        }

    return output
